/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration.exception;


/**
 * The sub-domain configuration is inconsistent:
 * <ul>
 * <li>an error was detected about the topology: the sub-domain mode is not supported,</li>
 * <li>or a new topology is not compliant, at sub-domain level, with the current one, defining a different sub-domain
 * mode URL.</li>
 * </ul>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class InconsistentSubdomainModeException extends InconsistentSubdomainConfigurationException {

    private static final long serialVersionUID = -5878089851234808198L;

    private final static String MESSAGE_TEMPLATE = "Inconsistent sub-domain mode for sub-domain '%s'";

    /**
     * The subdomain name having an unsupported or inconsistent mode
     */
    private final String subdomainName;

    public InconsistentSubdomainModeException(final String subdomainName) {
        super(String.format(MESSAGE_TEMPLATE, subdomainName));
        this.subdomainName = subdomainName;
    }

    /**
     * @return The subdomain name having an unsupported or inconsistent mode
     */
    public String getSubdomainName() {
        return this.subdomainName;
    }

}
