/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration.exception;

/**
 * A required server property is missing
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MissingServerPropertyException extends ConfigurationException {

    private static final long serialVersionUID = 4807857378710233406L;

    private static final String MESSAGE_FORMAT = "A required server property is missing or empty: %s";

    private static final String MESSAGE_EXTENDED_FORMAT = "A required server property is missing or empty: %s. %s";

    /**
     * The missing property name
     */
    private final String missingPropertyName;

    /**
     * @param missingPropertyName
     *            The missing property name
     */
    public MissingServerPropertyException(final String missingPropertyName) {
        super(String.format(MESSAGE_FORMAT, missingPropertyName));
        this.missingPropertyName = missingPropertyName;
    }

    /**
     * @param missingPropertyName
     *            The missing property name
     */
    public MissingServerPropertyException(final String missingPropertyName, final String descriptionMsg) {
        super(String.format(MESSAGE_EXTENDED_FORMAT, missingPropertyName, descriptionMsg));
        this.missingPropertyName = missingPropertyName;
    }

    /**
     * @return The missing property name
     */
    public String getMissingPropertyName() {
        return this.missingPropertyName;
    }

}
