/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container;

import java.util.logging.Logger;

import javax.naming.InitialContext;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;

/**
 * <p>
 * The ComponentContext needs to access some services such as the Router and so
 * on. Those services are implemented by Fractal components, that may
 * dynamically change during the runtime.
 * </p>
 * <p>
 * Thus, the ComponentContext can not reference directly those services. Each
 * time the ComponentContext uses those services, it retrieves through this
 * interface the good reference of the service. This interface is implemented by
 * the fractal components <code>Installer</code> and
 * <code>ComponentLifeCycle</code>.
 * </p>
 * 
 * @see org.ow2.petals.microkernel.container.lifecycle.Installer
 * @see org.ow2.petals.microkernel.container.lifecycle.ComponentLifeCycle
 * @author Adrien Louis - EBM WebSourcing
 * @author Nicolas Salatge - EBM WebSourcing
 */
public interface ComponentContextCommunication {

    /**
     * Gets the JBI Descriptor.
     * @return the JBI descriptor
     */
    Jbi getJBIDescriptor();

    /**
     * The installation directory of the component.
     * 
     * @return the installation root
     */
    String getInstallationRoot();

    /**
     * The working directory for the component.
     * 
     * @return the workspace root
     */
    String getWorkspaceRoot();

    /**
     * Get the Admin service.
     * 
     * @return the admin service
     */
    AdminService getAdminService();

    /**
     * Get the Container service.
     * 
     * @return the container service
     */
    ContainerService getContainerService();

    /**
     * The address of the component, as a Consumer.
     * 
     * @return the consumer endpoint
     */
    PetalsServiceEndpoint getAddress();

    /**
     * Get the EndpointRegistry service.
     * 
     * @return the endpoint service
     */
    EndpointRegistry getEndpointRegistry();

    /**
     * The Logger for the component.
     * 
     * @return the logger
     */
    Logger getLogger();

    /**
     * Get the Router service.
     * 
     * @return the router
     */
    RouterService getRouterService();

    /**
     * The Initial context for the component.
     * 
     * @return the initial context
     */
    InitialContext getInitialContext();

}
