/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container;

/**
 * A shared library identifier
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class SharedLibraryIdentifier {

    /**
     * Name part of the shared library identifier
     */
    private final String name;

    /**
     * Version part of the shared library identifier
     */
    private final String version;

    /**
     * Create a shared library identifier
     * 
     * @param name
     *            Name part of the shared library identifier
     * @param version
     *            Version part of the shared library identifier
     */
    public SharedLibraryIdentifier(final String name, String version) {
        this.name = name;
        this.version = version;
    }

    /**
     * @return Name part of the shared library identifier
     */
    public String getName() {
        return this.name;
    }

    /**
     * @return Version part of the shared library identifier
     */
    public String getVersion() {
        return this.version;
    }

    /*
     * Generated by Eclipse
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((name == null) ? 0 : name.hashCode());
        result = prime * result + ((version == null) ? 0 : version.hashCode());
        return result;
    }

    /*
     * Generated by Eclipse
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (!(obj instanceof SharedLibraryIdentifier)) {
            return false;
        }
        SharedLibraryIdentifier other = (SharedLibraryIdentifier) obj;
        if (name == null) {
            if (other.name != null) {
                return false;
            }
        } else if (!name.equals(other.name)) {
            return false;
        }
        if (version == null) {
            if (other.version != null) {
                return false;
            }
        } else if (!version.equals(other.version)) {
            return false;
        }
        return true;
    }
}