/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container;

import java.net.URL;
import java.util.List;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi.SharedLibrary;

/**
 * The <code>LifeCycle</code> interface of the shared library.
 * @author Nicolas SALATGE - EBM WebSourcing
 */
public interface SharedLibraryLifeCycle {

    public static final String COMPONENT_LOGGER_NAME = "Petals.Container.Lifecycle.SharedLibraryLifeCycle";

    /**
     * Name of the server interface of the Fractal component associated to the JBI shared library life cycle
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * Initialization of sharedLibraryLifeCycle.
     * 
     * @param sl
     *            the shared library
     * @param baseUrls
     *            the base urls
     * @param useParentFirst
     *            use parent first
     */
    void init(SharedLibrary sl, URL[] baseUrls, boolean useParentFirst);

    /**
     * Test the installation flag.
     * 
     * @return true if the shared library is already installed
     */
    boolean isLoaded();

    /**
     * Get the state
     * 
     * @return
     */
    String getState();

    SharedLibraryBean getSharedLibraryBean();

    /**
     * Install the jbi shared library. Call the Bootstrap onInstall() and
     * cleanup() methods, register the shared library object, and return its
     * jmxName.
     * 
     * @throws PetalsException
     *             the component is already installed
     */
    void loadSharedLibrary() throws PetalsException;

    /**
     * Uninstall the jbi shared library. Call the Bootstrap onUninstall() and
     * cleanup() methods, deregister the jbiComponent object.
     * 
     */
    void unLoadSharedLibrary();

    /**
     * Register a component which uses this Shared Library.
     * 
     * @param componentName
     */
    void registerComponent(String componentName);

    /**
     * Unregister a component if it is referenced by this Shared Library.
     * 
     * @param componentName
     */
    void unregisterComponent(String componentName);

    /**
     * get the list of the registered components.
     * 
     * @return the list
     */
    List<String> getRegisteredComponents();

}
