/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container.exception;


/**
 * The JBI component installer was not found
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class InstallerNotFoundException extends ContainerServiceException {

    private static final long serialVersionUID = -1629377691873888573L;

    private static final String MESSAGE_PATTERN = "The JBI component installer '%s' was not found.";

    /**
     * The JBI component identifier of the JBI component installer missing
     */
    private final String componentName;

    /**
     * @param componentName
     *            The JBI component identifier of the JBI component installer missing
     */
    public InstallerNotFoundException(final String componentName) {
        super(String.format(MESSAGE_PATTERN, componentName));
        this.componentName = componentName;
    }

    /**
     * @param componentName
     *            The JBI component identifier of the JBI component installer missing
     * @param cause
     *            The initial cause
     */
    public InstallerNotFoundException(final String componentName, final Throwable cause) {
        super(String.format(MESSAGE_PATTERN, componentName), cause);
        this.componentName = componentName;
    }

    /**
     * @return The JBI component identifier of the JBI component installer missing
     */
    public String getComponentName() {
        return this.componentName;
    }

}
