/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container.exception;


/**
 * The JBI service assembly life cycle was not found
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ServiceAssemblyLifecycleNotFoundException extends ContainerServiceException {

    private static final long serialVersionUID = 3908333621553895833L;

    private static final String MESSAGE_PATTERN = "The JBI service assembly life cycle '%s' was not found.";

    /**
     * The JBI service assembly identifier of the JBI service assembly life cycle missing
     */
    private final String saName;

    /**
     * @param saName
     *            The JBI service assembly identifier of the JBI service assembly life cycle missing
     */
    public ServiceAssemblyLifecycleNotFoundException(final String saName) {
        super(String.format(MESSAGE_PATTERN, saName));
        this.saName = saName;
    }

    /**
     * @param saName
     *            The JBI service assembly identifier of the JBI service assembly life cycle missing
     * @param cause
     *            The initial cause
     */
    public ServiceAssemblyLifecycleNotFoundException(final String saName, final Throwable cause) {
        super(String.format(MESSAGE_PATTERN, saName), cause);
        this.saName = saName;
    }

    /**
     * @return The JBI service assembly identifier of the JBI service assembly life cycle missing
     */
    public String getServiceAssemblyName() {
        return this.saName;
    }

}
