/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.extension;

import org.objectweb.fractal.api.Component;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.extension.exception.CreationExtensionException;
import org.ow2.petals.microkernel.api.extension.exception.PetalsExtensionException;

/**
 * <p>
 * Controller of an extension.
 * </p>
 * <p>
 * The controller aims to instantiate the Fractal component associated to the
 * extension and binds all interface.
 * </p>
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public interface PetalsExtensionController {

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the configuration service.
     */
    public final static String CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the admin service.
     */
    public final static String ADMIN_FRACTAL_ITF_NAME = "admin";

    /**
     * The name of the client interface of the Fractal composite "extension" of the container service.
     */
    public final static String CONTAINER_FRACTAL_ITF_NAME = "containerService";

    /**
     * The name of the client interface of the Fractal composite "extension" of the Petals admin service.
     */
    public final static String PETALSADMIN_FRACTAL_ITF_NAME = "petalsAdmin";

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the deployment service.
     */
    public final static String DEPLOYMENT_FRACTAL_ITF_NAME = "deployment";

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the atomic deployment service.
     */
    public final static String ATOMICDEPLOYMENT_FRACTAL_ITF_NAME = "atomicDeployment";

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the installation service.
     */
    public final static String INSTALLATION_FRACTAL_ITF_NAME = "installation";

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the component installation service.
     */
    public final static String COMPONENTINSTALLATION_FRACTAL_ITF_NAME = "componentInstallation";

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the JMX service.
     */
    public final static String JMX_FRACTAL_ITF_NAME = "jmx";

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the system state service.
     */
    public final static String SYSTEMSTATE_FRACTAL_ITF_NAME = "systemstate";

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the topology service.
     */
    public final static String TOPOLOGY_FRACTAL_ITF_NAME = "topology";

    /**
     * The name of the client interface of the Fractal composite "extension" of
     * the registry service.
     */
    public final static String REGISTRY_FRACTAL_ITF_NAME = "registry";

    /**
     * Name of the server interface of the Fractal component
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * Creates the Fractal component associated to the extension and binds all interfaces.
     * 
     * @param extensionComposite
     *            The Fractal component associated to the composite containing all extensions
     * @return The Fractal component.
     * @throws CreationExtensionException
     *             An error occurs creating the extension
     */
    public Component createFractalComponent(final Component extensionComposite)
            throws CreationExtensionException;

    /**
     * Remove a Fractal component associated to the extension and unbinds all
     * interfaces.
     */
    public void removeFractalComponent();
    
    /**
     * @return The name of the extension 
     */
    public String getExtensionName();

    /**
     * @param configurationService
     *            The configuration service to get the local container configuration.
     * @return <code>true</code> if the extension is enabled from the local container configuration, <code>false</code>
     *         if not.
     * @throws PetalsExtensionException
     *             An error occurs
     */
    public boolean isActivated(final ConfigurationService configurationService) throws PetalsExtensionException;

    /**
     * @return The dependencies on other extensions of the current extension.
     *         <code>null</code> or empty array is no dependency.
     */
    public String[] getDependencies();

    /**
     * Define what to do when an error occurs during the creation of the extension or when starting it.
     * 
     * @return <code>true</code> to fail and stop the Petals container startup, <code>false</code> to continue the
     *         Petals container startup
     */
    public boolean failsPetalsStartup();

}
