/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.implementation;

import java.util.Properties;

import org.ow2.petals.microkernel.api.implementation.exception.InvalidPropertyException;
import org.ow2.petals.microkernel.api.implementation.exception.MissingPropertyException;
import org.ow2.petals.microkernel.api.implementation.exception.MutableImplException;
import org.w3c.dom.Document;

/**
 * The base interface of all mutable implementations
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public interface MutableImplementation {

    /**
     * <p>
     * Retrieve the configuration of the mutable implementation as {@link Properties}. See API of the mutable
     * implementation for the property list.
     * </p>
     * <p>
     * All properties are prefixed by the class name of the mutable implementation
     * </p>
     * 
     * @param providedTopologyPassphrase
     *            Passphrase needed to return sensible information. If <code>null</code>, empty or not matching the one
     *            of the container on which the connection is established, no sensible information will be returned.
     * @return The configuration of the mutable implementation as {@link Properties}. Not <code>null</code>.
     * @throws MutableImplException
     *             An error occurs getting the configuration of the mutable implementation
     */
    public Properties getConfiguration(final String providedTopologyPassphrase) throws MutableImplException;

    /**
     * Get the XML part, located into the topology definition, of the mutable implementation configuration from a
     * property list generated by {@link #getConfiguration(String, String)}
     * 
     * @param properties
     *            A property list
     * @return The XML part, located into the topology definition, of the mutable implementation configuration, or
     *         <code>null</code> if:
     *         <ul>
     *         <li>the given parameter <code>properties</code> is null</li>
     *         <li>no configuration is required by the mutable implementation</li>
     *         </ul>
     * @throws MissingPropertyException
     *             A required property is missing.
     * @throws InvalidPropertyException
     *             A property has an invalid value.
     * @throws MutableImplException
     *             An error occurs creating the DOM {@link Document} associated to the topology part of the
     *             configuration
     */
    public Document getTopologyPartConfiguration(final Properties properties) throws MutableImplException;

}
