/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.implementation.exception;

import org.ow2.petals.microkernel.api.implementation.MutableImplementation;

/**
 * A property used by {@link MutableImplementation#getTopologyPartConfiguration(java.util.Properties)} has an invalid
 * valule.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class InvalidPropertyException extends MutableImplException {

    private static final long serialVersionUID = -4541167025429603684L;

    private static final String MESSAGE_PATTERN = "The property '%s' has an invalid value: '%s'";

    /**
     * The name of the invalid property
     */
    private final String propertyName;

    /**
     * The invalid value
     */
    private final String invalidValue;

    /**
     * @param propertyName
     *            The name of the invalid property
     */
    public InvalidPropertyException(final String propertyName, final String invalidValue) {
        super(String.format(MESSAGE_PATTERN, propertyName, invalidValue));
        this.propertyName = propertyName;
        this.invalidValue = invalidValue;
    }

    /**
     * @return The name of the invalid property
     */
    public String getPropertyName() {
        return this.propertyName;
    }

    /**
     * @return The invalid value
     */
    public String getInvalidValue() {
        return this.invalidValue;
    }

}
