/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.management;

import java.io.File;
import java.io.IOException;

import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;

/**
 * Utility class for handling packages and package URLs.
 * @author Olivier Fabre - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
public class PackageHelper {

    /**
     * Load the JBI descriptor from the repository folder of the given entity
     * (component, shared libraries or service assembly).
     * 
     * @param entityName
     *            name of entity
     * @param entityType
     *            type of entity (component, shared libraries or service
     *            assembly)
     * @return the {@link Jbi} object
     * @throws IOException
     * @throws JBIDescriptorException
     */
    public static final Jbi loadDescriptor(String entityName, JbiArchiveType entityType,
            RepositoryService repositoryService) throws JBIDescriptorException, IOException {

        final File folder;
        switch (entityType) {
            case COMPONENT_TYPE:
                folder = repositoryService.getComponentInstallDirectory(entityName);
                break;
            case SA_TYPE:
                folder = repositoryService.getServiceAssemblyInstallDirectory(entityName);
                break;
            case SL_TYPE:
                folder = repositoryService.getSharedLibraryInstallDirectory(entityName);
                break;
            default:
                throw new RuntimeException("Impossible case");
        }

        return JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptorFromFolder(folder);
    }

}
