/**
 * Copyright (c) 2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.messaging.exchange;

import java.io.IOException;
import java.sql.SQLException;

import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.api.system.persistence.PersistenceService;

/**
 * 
 * This abstract the specificities of the wrapper for persisting {@link PetalsMessageExchange}.
 * 
 * @author vnoel
 *
 */
public interface PersistedPetalsMessageExchange extends PetalsMessageExchange {

    /**
     * To persist the message (the in-memory version of it will be garbage collected).
     * 
     * @param persistenceService
     * @throws SQLException
     * @throws IOException
     */
    public void persistExchange(PersistenceService persistenceService) throws SQLException, IOException;

    public boolean isMessageExchangeStored();

    // TODO remove monitoring?
    public void setMonitored(final boolean monitored);

    /**
     * @return {@code true} if the message MUST be monitored (not if it actually is).
     */
    public boolean isMonitored();

    // TODO remove persistence?
    public void setPersisted(final boolean persisted);

    /**
     * @return {@code true} if the message MUST be persisted (not if it actually is).
     */
    public boolean isPersisted();

    /**
     * Redefine the return type. TODO move that to super interface? we can't because {@link PetalsServiceEndpoint} is
     * not visible from {@link PetalsMessageExchange}.
     */
    @Override
    public PetalsServiceEndpoint getConsumerEndpoint();

    /**
     * Redefine the return type. TODO move that to super interface? we can't because {@link PetalsServiceEndpoint} is
     * not visible from {@link PetalsMessageExchange}.
     */
    @Override
    public PetalsServiceEndpoint getEndpoint();
}
