/**
 * Copyright (c) 2014-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.util;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.TransportService;

/**
 * Utilities about {@link ConfigurationService}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ConfigurationServiceUtils {

    private ConfigurationServiceUtils() {
        // No constructor, it's an utility class
    }

    /**
     * Initializes a container configuration ({@link ContainerConfiguration}) bean from a container JAXB element (
     * {@link Container})
     * 
     * @param containerConfiguration
     *            The container configuration to initialize from the container <code>container</code>. Can not be
     *            {@code null}.
     * @param container
     *            A container configuration coming from its XML definition to use to initialize the container
     *            configuration <code>containerConfiguration</code>. Can not be <code>null</code>.
     * @param subdomainName
     *            The name of sub-domain whose the container is a member
     */
    public static void initializeContainerConfiguration(final ContainerConfiguration containerConfiguration,
            final Container container, final String subdomainName) {

        containerConfiguration.setName(container.getName());
        containerConfiguration.setDescription(container.getDescription());
        containerConfiguration.setUser(container.getUser());
        containerConfiguration.setPassword(container.getPassword());
        containerConfiguration.setHost(container.getHost());

        // JMX Service
        if (container.getJmxService() != null) {
            containerConfiguration.setJmxRMIConnectorPort(container.getJmxService().getRmiPort());
        }

        // Transport Service
        final TransportService transportService = container.getTransportService();
        if (transportService != null) {
            containerConfiguration.setTCPPort(transportService.getTcpPort());
        }

        containerConfiguration.setSubdomainName(subdomainName);
    }

}
