/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container.thread;

import static org.junit.Assert.fail;

import javax.jbi.JBIException;

import org.junit.Before;
import org.junit.Test;

/**
 * @author Marie Sauvage - EBM WebSourcing
 */
public class AbstractThreadTest {

    /**
     * The abstract thread to test
     */
    protected AbstractThread threadTest;

    /**
     * If the thread is not started, can not submit a task. The thread throws an
     * exception
     * 
     */
    @Test
    public void testSubmissionFailure() {
        try {
            threadTest.execute("TASK");
            fail();
        } catch (JBIException e) {
        }
    }

    /**
     * Test multiple task submitter in separate threads.
     * 
     */
    @Test
    public void testMultipleConsumers() {
        int loopA = 10;
        long sleepA = 100;
        int loopB = 5;
        long sleepB = 120;

        threadTest.start();

        ConsumerThread consumerA = new ConsumerThread(threadTest, loopA, sleepA);
        ConsumerThread consumerB = new ConsumerThread(threadTest, loopB, sleepB);

        consumerA.start();
        consumerB.start();

        // wait for tasks to complete
        try {
            consumerA.join(loopA * sleepA * 2);
            consumerB.join(loopB * sleepB * 2);
        } catch (InterruptedException e) {
            fail("Task submission failure");
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @Before
    public void setUp() throws Exception {
        /*
         * Create the thread test class overriding the doTask method
         */
        threadTest = new AbstractThread(0, null) {
            @Override
            protected boolean doTask(String action) {
                boolean result = false;
                if (action.equals("SHUTDOWN")) {
                    result = false;
                } else {
                    result = true;
                }
                return result;
            }
        };
    }

    /**
     * 
     * @author Christophe Hamerling - EBM WebSourcing
     * 
     */
    protected static class ConsumerThread extends Thread {

        private AbstractThread abstractThread;

        private long sleepTime;

        private int loop;

        /**
         * Creates a new instance of ConsumerThread
         * 
         * @param abstractThread
         * @param loop
         * @param sleepTime
         */
        public ConsumerThread(AbstractThread abstractThread, int loop, long sleepTime) {
            this.abstractThread = abstractThread;
            this.loop = loop;
            this.sleepTime = sleepTime;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Thread#run()
         */
        @Override
        public void run() {
            for (int i = 0; i < loop; i++) {
                try {
                    abstractThread.execute("" + i);
                } catch (JBIException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                }
                try {
                    Thread.sleep(sleepTime);
                } catch (InterruptedException e) {
                }
            }
        }
    }
}
